import logging
import re

from django.conf import settings

from rich.console import Console
from rich.live import Live
from rich.table import Table
from rich.text import Text


class DjangoRequestHandler(logging.StreamHandler):
    """
    A logging handler that prints Django requests in a live-updating table.
    """

    live = None
    uri_table = None
    console = None

    def __init__(self, *args, **kwargs):
        super().__init__()

        if "console" in kwargs:
            self.console = kwargs["console"]

        if self.console is None:
            self.console = Console()

        if self.live is None:
            self.uri_table = Table()
            self.uri_table.add_column("Method")
            self.uri_table.add_column("URI")
            self.uri_table.add_column("Status")
            self.uri_table.add_column("Size")

            self.live = Live(self.uri_table, auto_refresh=False)

    def emit(self, record):
        # Ignore any message from `django.server` that don't have 3 args.
        # There might be a better approach to this, not sure.
        if record.name != "django.server" or len(record.args) != 3:
            super().emit(record)
            return

        try:
            request = record.args[0]
            status = record.args[1]
            content_length = record.args[2]

            # Example: GET /static/unicorn/js/messageSender.js HTTP/1.1
            matches = re.match(
                r"(GET|POST|PATCH|UPDATE|HEAD|PUT|DELETE|CONNECT|OPTIONS|TRACE)\s+([\S]+)\s+([\S]+)",
                request,
            )

            if matches:
                method = matches.group(1)
                uri = matches.group(2)

                if not settings.STATIC_URL or not uri.startswith(settings.STATIC_URL):
                    method_style = "green"
                    status_style = "green"

                    if status.startswith("3"):
                        method_style = "yellow"
                        status_style = "yellow"
                    if status.startswith("4"):
                        method_style = "orange"
                        status_style = "orange"
                    if status.startswith("5"):
                        method_style = "red"
                        status_style = "red"

                    self.uri_table.add_row(
                        Text(method, style=method_style),
                        Text(uri, style="white bold"),
                        Text(status, style=status_style),
                        Text(content_length),
                    )
                    self.live.start()
                    self.live.refresh()

            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception as ex:
            # self.handleError(record)
            print(ex)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_rich_logging',
 'django_rich_logging.management',
 'django_rich_logging.management.commands']

package_data = \
{'': ['*']}

install_requires = \
['Django>3.0', 'rich>=11.2.0,<12.0.0']

extras_require = \
{'docs': ['Sphinx>=4.3.2,<5.0.0',
          'linkify-it-py>=1.0.3,<2.0.0',
          'myst-parser>=0.16.1,<0.17.0',
          'furo>=2021.11.23,<2022.0.0',
          'sphinx-copybutton>=0.4.0,<0.5.0',
          'sphinx-autobuild>=2021.3.14,<2022.0.0',
          'toml']}

setup_kwargs = {
    'name': 'django-rich-logging',
    'version': '0.1.0',
    'description': 'A prettier way to see Django requests while developing.',
    'long_description': '<p align="center">\n  <a href="https://django-rich-logging.readthedocs.io"><h1 align="center">django-rich-logging</h1></a>\n</p>\n<p align="center">A prettier way to see Django requests while developing.</p>\n\n![PyPI](https://img.shields.io/pypi/v/django-rich-logging?color=blue&style=flat-square)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/django-rich-logging?color=blue&style=flat-square)\n![GitHub Sponsors](https://img.shields.io/github/sponsors/adamghill?color=blue&style=flat-square)\n\n📖 Complete documentation: https://django-rich-logging.readthedocs.io\n\n📦 Package located at https://pypi.org/project/django-rich-logging/\n\n## ⭐ Features\n\n- live-updating table of all requests while developing\n\n## Installation\n\n`poetry add django-rich-logging` OR `pip install django-rich-logging`\n\n### Configuration\n\n```python\n# settings.py\n\n# other settings here\n\nLOGGING = {\n    "version": 1,\n    "disable_existing_loggers": False,\n    "handlers": {\n        "django_rich_logging": {\n            "class": "django_rich_logging.logging.DjangoRequestHandler",\n            "level": "INFO",\n        },\n    },\n    "loggers": {\n        "django.server": {"handlers": ["django_rich_logging"], "level": "INFO"},\n    },\n}\n\n# other settings here\n```\n\nRead all of the documentation at https://django-rich-logging.readthedocs.io.\n',
    'author': 'adamghill',
    'author_email': 'adam@adamghill.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/adamghill/django-rich-logging/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
